@@ -27,6 +27,56 @@
 
 sen_ctx sen_gctx;
 
+/* simple array */
+
+void
+sen_array_init(sen_array *a, sen_ctx *ctx, uint16_t element_size, uint16_t flags)
+{
+  a->ctx = ctx;
+  a->element_size = element_size;
+  a->flags = flags;
+  a->max = 0;
+  if (flags & SEN_ARRAY_THREADSAFE) { MUTEX_INIT(a->lock); }
+  memset(a->elements, 0, sizeof(a->elements));
+}
+
+void
+sen_array_fin(sen_array *a)
+{
+  int i;
+  sen_ctx *ctx = a->ctx;
+  for (i = 0; i < 8; i++) {
+    void **e = &a->elements[i];
+    if (*e) {
+      SEN_FREE(*e);
+      *e = NULL;
+    }
+  }
+}
+
+void *
+sen_array_at(sen_array *a, sen_id id)
+{
+  void *e;
+  SEN_ARRAY_AT(a, id, e);
+  return e;
+}
+
+sen_id
+sen_array_id(sen_array *a, void *p)
+{
+  uint32_t i, s, n;
+  uintptr_t o, p_ = (uintptr_t) p;
+  for (i = 0, s = 0; i < 8; i++, s += n) {
+    n = 0x0f * (1 << (i << 2));
+    o = (uintptr_t) a->elements[i];
+    if (o <= p_ && p_ < o + n * a->element_size) {
+      return s + ((p_ - o) / a->element_size);
+    }
+  }
+  return SEN_SYM_NIL;
+}
+
 /* fixme by 2038 */
 
 sen_rc
@@ -72,25 +122,47 @@ sen_timeval2str(sen_timeval *tv, char *buf)
 }
 
 sen_rc
-sen_str2timeval(const char *str, uint32_t str_len, const char *format, sen_timeval *tv)
+sen_str2timeval(const char *str, uint32_t str_len, sen_timeval *tv)
 {
   struct tm tm;
-  const char *r1, *r2;
-  char buf[SEN_TIMEVAL_STR_SIZE];
-  uint32_t uv, len = str_len >= SEN_TIMEVAL_STR_SIZE ? SEN_TIMEVAL_STR_SIZE - 1 : str_len;
-  memcpy(buf, str, len);
-  buf[len] = '\0';
+  const char *r1, *r2, *rend = str + str_len;
+  uint32_t uv;
   memset(&tm, 0, sizeof(struct tm));
-  if (!format) { format = SEN_TIMEVAL_STRP_FORMAT; }
-  if (!(r1 = strptime(buf, format, &tm))) {
-    if ((format != SEN_TIMEVAL_STRP_FORMAT) ||
-        !(r1 = strptime(buf, "%Y/%m/%d %H:%M:%S", &tm))) {
-      return sen_invalid_argument;
-    }
+
+  tm.tm_year = (int)sen_atoui(str, rend, &r1) - 1900;
+  if ((r1 + 1) >= rend || (*r1 != '/' && *r1 != '-') ||
+      tm.tm_year < 0) { return sen_invalid_argument; }
+  r1++;
+  tm.tm_mon = (int)sen_atoui(r1, rend, &r1) - 1;
+  if ((r1 + 1) >= rend || (*r1 != '/' && *r1 != '-') ||
+      tm.tm_mon < 0 || tm.tm_mon >= 12) { return sen_invalid_argument; }
+  r1++;
+  tm.tm_mday = (int)sen_atoui(r1, rend, &r1);
+  if ((r1 + 1) >= rend || *r1 != ' ' ||
+      tm.tm_mday < 1 || tm.tm_mday > 31) { return sen_invalid_argument; }
+
+  tm.tm_hour = (int)sen_atoui(++r1, rend, &r2);
+  if ((r2 + 1) >= rend || r1 == r2 || *r2 != ':' ||
+      tm.tm_hour < 0 || tm.tm_hour >= 24) {
+    return sen_invalid_argument;
   }
+  r1 = r2 + 1;
+  tm.tm_min = (int)sen_atoui(r1, rend, &r2);
+  if ((r2 + 1) >= rend || r1 == r2 || *r2 != ':' ||
+      tm.tm_min < 0 || tm.tm_min >= 60) {
+    return sen_invalid_argument;
+  }
+  r1 = r2 + 1;
+  tm.tm_sec = (int)sen_atoui(r1, rend, &r2);
+  if (r1 == r2 ||
+      tm.tm_sec < 0 || tm.tm_sec > 61 /* leap 2sec */) {
+    return sen_invalid_argument;
+  }
+  r1 = r2;
+
   if ((tv->tv_sec = (int32_t) mktime(&tm)) == -1) { return sen_invalid_argument; }
-  if (*r1 == '.') { r1++; }
-  uv = sen_atoi(r1, r1 + strlen(r1), &r2);
+  if ((r1 + 1) < rend && *r1 == '.') { r1++; }
+  uv = sen_atoi(r1, rend, &r2);
   while (r2 < r1 + 6) {
     uv *= 10;
     r2++;
@@ -133,7 +205,12 @@ sen_ctx_init(sen_ctx *ctx)
   ctx->inbuf = NULL;
   ctx->co.mode = 0;
   ctx->co.func = NULL;
+#ifdef USE_SET_AS_OBJECTS
   ctx->objects = NULL;
+#else /* USE_SET_AS_OBJECTS */
+  sen_array_init(&ctx->objects, ctx, sizeof(sen_obj), 0);
+  ctx->freelist = NIL;
+#endif /* USE_SET_AS_OBJECTS */
   ctx->symbols = NULL;
   ctx->com = NULL;
   sen_rbuf_init(&ctx->outbuf, 0);
@@ -144,6 +221,7 @@ sen_rc
 sen_ctx_fin(sen_ctx *ctx)
 {
   sen_rc rc = sen_success;
+#ifdef USE_SET_AS_OBJECTS
   if (ctx->objects) {
     sen_obj *o;
     sen_set_cursor *sc;
@@ -155,6 +233,16 @@ sen_ctx_fin(sen_ctx *ctx)
     }
     sen_set_close(ctx->objects);
   }
+#else /* USE_SET_AS_OBJECTS */
+  {
+    sen_id id;
+    sen_obj *o;
+    SEN_ARRAY_EACH(&ctx->objects, 1, ctx->seqno, id, o, {
+      if (!(o->flags & SEN_OBJ_FREE)) { sen_obj_clear(ctx, o); }
+    });
+  }
+  sen_array_fin(&ctx->objects);
+#endif /* USE_SET_AS_OBJECTS */
   if (ctx->symbols) {
     sen_set_close(ctx->symbols);
   }
@@ -177,8 +265,10 @@ sen_ctx_fin(sen_ctx *ctx)
 void
 sen_ctx_initql(sen_ctx *ctx)
 {
-  if (!ctx->objects && !ctx->symbols) {
+  if (!ctx->symbols) {
+#ifdef USE_SET_AS_OBJECTS
     if ((ctx->objects = sen_set_open(sizeof(int), sizeof(sen_obj), 0))) {
+#endif /* USE_SET_AS_OBJECTS */
       if ((ctx->symbols = sen_set_open(0, sizeof(sen_obj), 0))) {
         if (ctx->db) { sen_ql_def_db_funcs(ctx); }
         if (!ERRP(ctx, SEN_ERROR)) {
@@ -192,11 +282,13 @@ sen_ctx_initql(sen_ctx *ctx)
       } else {
         MERR("ctx->symbols init failed");
       }
+#ifdef USE_SET_AS_OBJECTS
       sen_set_close(ctx->objects);
       ctx->objects = NULL;
     } else {
       MERR("ctx->objects init failed");
     }
+#endif /* USE_SET_AS_OBJECTS */
   } else {
     MERR("invalid ctx assigned");
   }
@@ -279,15 +371,18 @@ sen_init(void)
   return rc;
 }
 
+static int alloc_count = 0;
+
 sen_rc
 sen_fin(void)
 {
-  SEN_LOG(sen_log_notice, "sen_fin");
   sen_ctx_fin(&sen_gctx);
   sen_lex_fin();
-  sen_logger_fin();
+  sen_str_fin();
   sen_com_fin();
-  return sen_str_fin();
+  SEN_LOG(sen_log_notice, "sen_fin (%d)", alloc_count);
+  sen_logger_fin();
+  return sen_success;
 }
 
 sen_ctx *
@@ -352,14 +447,14 @@ sen_ctx_send(sen_ctx *ctx, char *str, unsigned int str_len, int flags)
     }
     goto exit;
   } else {
-    if (ctx->objects) {
+    if (ctx->symbols) {
       sen_ql_feed(ctx, str, str_len, flags);
+      if (ctx->stat == SEN_QL_QUITTING) { ctx->stat = SEN_CTX_QUIT; }
       if (!ERRP(ctx, SEN_CRIT)) {
         if (!(flags & SEN_CTX_QUIET) && ctx->output) {
           ctx->output(ctx, 0, ctx->data.ptr);
         }
       }
-      if (ctx->stat == SEN_QL_QUITTING) { ctx->stat = SEN_CTX_QUIT; }
       goto exit;
     }
   }
@@ -397,10 +492,10 @@ sen_ctx_recv(sen_ctx *ctx, char **str, unsigned int *str_len, int *flags)
     }
     goto exit;
   } else {
-    if (ctx->objects) {
+    if (ctx->symbols) {
       sen_rbuf *buf = &ctx->outbuf;
       unsigned int head, tail;
-      int *offsets = (unsigned int *) ctx->subbuf.head;
+      unsigned int *offsets = (unsigned int *) ctx->subbuf.head;
       int npackets = SEN_RBUF_VSIZE(&ctx->subbuf) / sizeof(unsigned int);
       if (npackets < ctx->bufcur) { return sen_invalid_argument; }
       head = ctx->bufcur ? offsets[ctx->bufcur - 1] : 0;
@@ -470,7 +565,10 @@ sen_obj *
 sen_get(const char *key)
 {
   sen_obj *obj;
-  if (!sen_set_get(sen_gctx.symbols, key, (void **) &obj)) { return F; }
+  if (!sen_set_get(sen_gctx.symbols, key, (void **) &obj)) {
+    SEN_LOG(sen_log_warning, "sen_get(%s) failed", key);
+    return F;
+  }
   if (!obj->flags) {
     obj->flags |= SEN_OBJ_SYMBOL;
     obj->type = sen_ql_void;
@@ -492,6 +590,7 @@ sen_del(const char *key)
   sen_obj *obj;
   sen_set_eh *eh;
   if (!(eh = sen_set_get(sen_gctx.symbols, key, (void **) &obj))) {
+    SEN_LOG(sen_log_warning, "sen_del(%s) failed", key);
     return sen_invalid_argument;
   }
   /* todo : do something */
@@ -500,8 +599,6 @@ sen_del(const char *key)
 
 /**** memory allocation ****/
 
-static int alloc_count = 0;
-
 void *
 sen_malloc(sen_ctx *ctx, size_t size, const char* file, int line)
 {
@@ -654,6 +751,7 @@ sen_obj *
 sen_obj_new(sen_ctx *ctx)
 {
   sen_obj *o;
+#ifdef USE_SET_AS_OBJECTS
   do {
     if (!sen_set_get(ctx->objects, &ctx->seqno, (void **) &o)) {
       MERR("sen_set_get failed");
@@ -662,8 +760,21 @@ sen_obj_new(sen_ctx *ctx)
     // SEN_SET_INT_ADD(ctx->objects, &ctx->seqno, o);
     ctx->seqno++;
   } while (o->type);
+#else /* USE_SET_AS_OBJECTS */
+  if (ctx->freelist == NIL) {
+    if (!(o = sen_array_at(&ctx->objects, ctx->seqno + 1))) {
+      MERR("sen_set_get failed");
+      return NULL;
+    }
+    ctx->seqno++;
+  } else {
+    o = ctx->freelist;
+    ctx->freelist = CDR(ctx->freelist);
+  }
+#endif /* USE_SET_AS_OBJECTS */
   o->flags = 0;
   o->nrefs = 0;
+  ctx->n_entries++;
   return o;
 }
 
@@ -741,7 +852,11 @@ sen_obj_clear(sen_ctx *ctx, sen_obj *o)
     case sen_ql_query :
       if (o->u.p.value) { sen_query_close(o->u.p.value); }
       break;
+    case sen_ql_symsnip :
+      sen_ql_symsnip_spec_close(ctx, (symsnip_spec *)o->u.p.value);
+      break;
     default :
+      SEN_LOG(sen_log_warning, "obj_clear: invalid type(%x)", o->type);
       break;
     }
   }
