@@ -21,10 +21,6 @@
 #include <errno.h>
 #endif /* HAVE_ERRNO_H */
 
-#ifndef SEN_STORE_H
-#include "store.h"
-#endif /* SEN_STORE_H */
-
 #ifndef SEN_COM_H
 #include "com.h"
 #endif /* SEN_COM_H */
@@ -147,8 +143,8 @@ typedef sen_obj *sen_ql_native_func(sen_ctx *, sen_obj *, sen_ql_co *);
 
 struct _sen_obj {
   uint8_t type;
-  uint8_t flags;
-  uint16_t nrefs;
+  uint8_t nrefs;
+  uint16_t flags;
   sen_id class;
   union {
     struct {
@@ -190,10 +186,83 @@ struct _sen_ql_co {
   void *data;
 };
 
+/**** sen_array ****/
+
+typedef struct _sen_array sen_array;
+
+enum {
+  SEN_ARRAY_CLEAR = 1,
+  SEN_ARRAY_THREADSAFE = 2
+};
+
+struct _sen_array {
+  sen_ctx *ctx;
+  sen_id max;
+  uint16_t element_size;
+  uint16_t flags;
+  sen_mutex lock;
+  void *elements[8];
+};
+
+#define SEN_ARRAY_EACH(a,head,tail,key,val,block) do {\
+  int _ei;\
+  const sen_id h = (head);\
+  const sen_id t = (tail);\
+  for (_ei = 0, (key) = (h); _ei < 8 && (key) <= (t); _ei++) {\
+    int _ej = 0x0f * (1 << (_ei << 2));\
+    if (((val) = (a)->elements[_ei])) {\
+      for (; _ej-- && (key) <= (t); (key)++, (val) = (void *)((byte *)(val) + (a)->element_size)) block\
+    } else {\
+      (key) += _ej;\
+    }\
+  }\
+} while (0)
+
+#define SEN_ARRAY_AT(a,id,e) do {\
+  sen_id id_ = (id);\
+  byte **e_;\
+  size_t o_;\
+  {\
+    int l_, i_;\
+    SEN_BIT_SCAN_REV(id_, l_);\
+    i_ = l_ >> 2;\
+    o_ = 1 << (i_ << 2);\
+    e_ = (byte **)&(a)->elements[i_];\
+  }\
+  if (!*e_) {\
+    sen_ctx *ctx = (a)->ctx;\
+    if ((a)->flags & SEN_ARRAY_THREADSAFE) { MUTEX_LOCK((a)->lock); }\
+    if (!*e_) {\
+      if ((a)->flags & SEN_ARRAY_CLEAR) {\
+        *e_ = SEN_CALLOC(0x0f * o_ * (a)->element_size);\
+      } else {\
+        *e_ = SEN_MALLOC(0x0f * o_ * (a)->element_size);\
+      }\
+    }\
+    if ((a)->flags & SEN_ARRAY_THREADSAFE) { MUTEX_UNLOCK((a)->lock); }\
+    if (!*e_) { e = NULL; break; }\
+  }\
+  if (id_ > (a)->max) { (a)->max = id_; }\
+  (e) = (void *)(*e_ + (id_ - o_) * (a)->element_size);\
+} while (0)
+
+#define SEN_ARRAY_NEXT(a,e) SEN_ARRAY_AT((a), (a)->max + 1, e)
+
+void sen_array_init(sen_array *a, sen_ctx *ctx, uint16_t element_size, uint16_t flags);
+void sen_array_fin(sen_array *a);
+void *sen_array_at(sen_array *a, sen_id id);
+sen_id sen_array_id(sen_array *a, void *p);
+
 /**** sen_ctx ****/
 
+#ifndef SEN_STORE_H
+#include "store.h"
+#endif /* SEN_STORE_H */
+
 #define SEN_CTX_MSGSIZE 128
 
+// #define USE_SET_AS_OBJECTS
+
 struct _sen_ctx {
   sen_rc rc;
   uint8_t errlvl;
@@ -223,7 +292,13 @@ struct _sen_ctx {
 
   sen_obj arg;         /* wait_data container (for coroutine) */
   sen_db *db;
+  uint32_t n_entries;
+#ifdef USE_SET_AS_OBJECTS
   sen_set *objects;    /* object table */
+#else /* USE_SET_AS_OBJECTS */
+  sen_array objects;   /* object table */
+  sen_obj *freelist;   /* object free list */
+#endif /* USE_SET_AS_OBJECTS */
   sen_set *symbols;    /* symbol table */
   sen_obj *phs;        /* list of placeholders */
   sen_ql_co co;        /* coroutine info */
@@ -234,6 +309,7 @@ struct _sen_ctx {
   sen_obj *dump;       /* stack register for next evaluation */
   sen_obj *value;      /* evaluated value */
   sen_obj *global_env; /* global variables */
+
   uint8_t op;
 
   char *inbuf;
@@ -270,13 +346,10 @@ sen_rc sen_del(const char *key);
 #ifndef SEN_TIMEVAL_STR_FORMAT
 #define SEN_TIMEVAL_STR_FORMAT "%04d-%02d-%02d %02d:%02d:%02d.%06d"
 #endif /* SEN_TIMEVAL_STR_FORMAT */
-#ifndef SEN_TIMEVAL_STRP_FORMAT
-#define SEN_TIMEVAL_STRP_FORMAT "%Y-%m-%d %H:%M:%S"
-#endif /* SEN_TIMEVAL_STR_FORMAT */
 
 sen_rc sen_timeval_now(sen_timeval *tv);
 sen_rc sen_timeval2str(sen_timeval *tv, char *buf);
-sen_rc sen_str2timeval(const char *str, uint32_t str_len, const char *format, sen_timeval *tv);
+sen_rc sen_str2timeval(const char *str, uint32_t str_len, sen_timeval *tv);
 
 void sen_ctx_log(sen_ctx *ctx, char *fmt, ...);
 
