@@ -65,9 +65,9 @@ sen_inv *sen_inv_open(const char *path, sen_sym *lexicon);
 sen_rc sen_inv_close(sen_inv *inv);
 sen_rc sen_inv_remove(const char *path);
 sen_rc sen_inv_info(sen_inv *inv, uint64_t *seg_size, uint64_t *chunk_size);
-sen_rc sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h,
+sen_rc sen_inv_update_one(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h,
                       int hint);
-sen_rc sen_inv_delete(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h);
+sen_rc sen_inv_delete_one(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h);
 uint32_t sen_inv_initial_n_segments(sen_inv *inv);
 
 sen_inv_updspec *sen_inv_updspec_open(uint32_t rid, uint32_t sid);
@@ -78,6 +78,7 @@ int sen_inv_updspec_cmp(sen_inv_updspec *a, sen_inv_updspec *b);
 uint32_t sen_inv_estimate_size(sen_inv *inv, uint32_t key);
 
 void sen_inv_seg_expire(sen_inv *inv, int32_t threshold);
+void sen_inv_expire(void);
 
 typedef struct {
   sen_id rid;
@@ -104,6 +105,7 @@ typedef struct {
   uint16_t buffer_pseg;
   uint16_t with_pos;
   int flags;
+  uint16_t *ppseg;
 } sen_inv_cursor;
 
 #define SEN_INV_CURSOR_CMP(c1,c2) \
@@ -123,6 +125,24 @@ uint32_t sen_inv_max_section(sen_inv *inv);
 
 int sen_inv_check(sen_inv *inv);
 const char *sen_inv_path(sen_inv *inv);
+sen_sym *sen_inv_lexicon(sen_inv *inv);
+
+sen_rc sen_inv_upd(sen_inv *inv, sen_id rid, sen_vgram *vgram,
+                   const char *oldvalue, unsigned int oldvalue_len,
+                   const char *newvalue, unsigned int newvalue_len);
+sen_rc sen_inv_update(sen_inv *inv, sen_id rid, sen_vgram *vgram,
+                      unsigned int section,
+                      sen_values *oldvalues, sen_values *newvalues);
+sen_rc sen_inv_term_extract(sen_inv *inv, const char *string,
+                            unsigned int string_len, sen_set *s,
+                            sen_sel_operator op, sen_select_optarg *optarg);
+sen_rc sen_inv_similar_search(sen_inv *inv, const char *string, unsigned int string_len,
+                              sen_set *s, sen_sel_operator op, sen_select_optarg *optarg);
+sen_rc sen_inv_select(sen_inv *inv, const char *string, unsigned int string_len,
+                      sen_set *s, sen_sel_operator op, sen_select_optarg *optarg);
+sen_set *sen_inv_sel(sen_inv *inv, const char *string, unsigned int string_len);
+
+sen_rc sen_query_select(sen_inv *i, sen_query *q, sen_records *r, sen_sel_operator op);
 
 #ifdef __cplusplus
 }
