@@ -1109,7 +1109,7 @@ sen_inv_open08(const char *path, sen_sym *lexicon)
 }
 
 sen_rc
-sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int hint)
+sen_inv_update_one08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int hint)
 {
   sen_rc r = sen_success;
   buffer *b;
@@ -1117,7 +1117,7 @@ sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int
   buffer_rec *br = NULL;
   buffer_term *bt;
   uint32_t pos = 0, size, *a;
-  if (!u->tf || !u->sid) { return sen_inv_delete(inv, key, u, h); }
+  if (!u->tf || !u->sid) { return sen_inv_delete_one(inv, key, u, h); }
   if (!(a = array_get(inv, key))) { return sen_memory_exhausted; }
   if (!(bs = encode_rec(u, &size, 0))) { r = sen_memory_exhausted; goto exit; }
   for (;;) {
@@ -1132,7 +1132,7 @@ sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int
           buffer_close(inv, pos);
           if ((r = buffer_flush(inv, pos >> W_OF_SEGMENT, h))) { goto exit; }
           if (*a != pos) {
-            SEN_LOG(sen_log_debug, "sen_inv_update: *a changed %d->%d", *a, pos);
+            SEN_LOG(sen_log_debug, "sen_inv_update_one: *a changed %d->%d", *a, pos);
             continue;
           }
           if ((r = buffer_open(inv, pos, &bt, &b))) {
@@ -1144,7 +1144,7 @@ sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int
                   b->header.nterms, b->header.nterms_void);
           if (b->header.buffer_free < size) {
             buffer_close(inv, pos);
-            SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_update",
+            SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_update_one",
                     *a, b->header.buffer_free, size);
             /* todo: must be splitted */
             r = sen_memory_exhausted;
@@ -1168,7 +1168,7 @@ sen_inv_update08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h, int
         if (u2.rid != u->rid || u2.sid != u->sid) {
           uint8_t *bs2 = encode_rec(&u2, &size2, 0);
     if (!bs2) {
-      SEN_LOG(sen_log_alert, "encode_rec on sen_inv_update failed !");
+      SEN_LOG(sen_log_alert, "encode_rec on sen_inv_update_one failed !");
       r = sen_memory_exhausted; 
       goto exit;
     }
@@ -1221,7 +1221,7 @@ exit :
 }
 
 sen_rc
-sen_inv_delete08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h)
+sen_inv_delete_one08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h)
 {
   sen_rc r = sen_success;
   buffer *b;
@@ -1253,14 +1253,14 @@ sen_inv_delete08(sen_inv *inv, uint32_t key, sen_inv_updspec *u, sen_set *h)
       buffer_close(inv, *a);
       if ((r = buffer_flush(inv, *a >> W_OF_SEGMENT, h))) { goto exit; }
       if (*a != _a) {
-        SEN_LOG(sen_log_debug, "sen_inv_delete: *a changed %d->%d)", *a, _a);
+        SEN_LOG(sen_log_debug, "sen_inv_delete_one: *a changed %d->%d)", *a, _a);
         continue;
       }
       if ((r = buffer_open(inv, *a, &bt, &b))) { goto exit; }
       SEN_LOG(sen_log_debug, "flushed!  b=%p free=%d, seg(%d)", b, b->header.buffer_free, *a >> W_OF_SEGMENT);
       if (b->header.buffer_free < size) {
         /* todo: must be splitted ? */
-        SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_delete",
+        SEN_LOG(sen_log_crit, "buffer(%d) is full (%d < %d) in sen_inv_delete_one",
                 *a, b->header.buffer_free, size);
         r = sen_memory_exhausted;
         buffer_close(inv, *a);
