@@ -5,7 +5,7 @@
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
 
-nnn  This library is distributed in the hope that it will be useful,
+  This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
@@ -21,6 +21,7 @@ nnn  This library is distributed in the hope that it will be useful,
 #include <ctype.h>
 #include "snip.h"
 #include "sym.h"
+#include "inv.h"
 #include "ql.h"
 
 /* query string parser and executor */
@@ -482,14 +483,19 @@ sen_query_close(sen_query *q)
 }
 
 static void
-exec_query(sen_index *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator op)
+exec_query(sen_inv *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator op)
 {
   sen_records *s;
   cell *e, *ope = NIL;
   int n = sen_records_nhits(r);
   sen_sel_operator op0 = sen_sel_or, *opp = &op0, op1 = q->default_op;
   if (!n && op != sen_sel_or) { return; }
-  s = n ? sen_records_open(r->record_unit, r->subrec_unit, 0) : r;
+  if (n) {
+    s = sen_records_open(r->record_unit, r->subrec_unit, 0);
+    s->keys = r->keys;
+  } else {
+    s = r;
+  }
   while (c != NIL) {
     POP(e, c);
     switch (e->type) {
@@ -519,8 +525,8 @@ exec_query(sen_index *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator
           q->opt.vector_size = DEFAULT_WEIGHT + q->weight_offset;
         }
       }
-      if (sen_index_select(i, e->u.b.value, e->u.b.size, s, *opp, &q->opt)) {
-        SEN_LOG(sen_log_error, "sen_index_select on exec_query failed !");
+      if (sen_inv_select(i, e->u.b.value, e->u.b.size, s, *opp, &q->opt)) {
+        SEN_LOG(sen_log_error, "sen_inv_select on exec_query failed !");
         return;
       }
       break;
@@ -558,11 +564,9 @@ exec_query(sen_index *i, sen_query *q, cell *c, sen_records *r, sen_sel_operator
 }
 
 sen_rc
-sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op)
+sen_query_select(sen_inv *i, sen_query *q, sen_records *r, sen_sel_operator op)
 {
-  int p;
-  if (!i || !q || !r || !PAIRP(q->expr)) { return sen_invalid_argument; }
-  p = q->escalation_threshold;
+  int p = q->escalation_threshold;
   // dump_query(q, q->expr, 0);
   // sen_log("escalation_threshold=%d", p);
   if (p >= 0 || (-p & 1)) {
@@ -585,6 +589,17 @@ sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op)
   return sen_success;
 }
 
+sen_rc
+sen_query_exec(sen_index *i, sen_query *q, sen_records *r, sen_sel_operator op)
+{
+  sen_rc rc;
+  if (!i || !q || !r || !PAIRP(q->expr)) { return sen_invalid_argument; }
+  r->keys = i->keys;
+  rc = sen_query_select(i->inv, q, r, op);
+  sen_records_cursor_clear(r);
+  return rc;
+}
+
 static int
 query_term_rec(sen_query* q, cell* c, query_term_callback func, void *func_arg)
 {
